"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.failureDescriber = exports.explanation = void 0;
const lines_unlines_1 = require("lines-unlines");
const INDENTATION = "  ";
function formatDependency(d) {
    return INDENTATION + d.key + ": " + d.selector;
}
function explanation(failure) {
    switch (failure.result.reason) {
        case 0:
            return lines_unlines_1.unlines([
                `These dependencies were not found:`,
                ``,
                lines_unlines_1.unlines(failure.result.dependencies.map(formatDependency)),
            ]);
        case 1:
            return lines_unlines_1.unlines([
                `The operation failed with this error:`,
                ``,
                failure.result.message,
            ]);
    }
}
exports.explanation = explanation;
function failureDescriber(context) {
    return failure => lines_unlines_1.unlines([
        `Could not ${failure.operation.description} on this page:`,
        ``,
        INDENTATION + location.href,
        ``,
        explanation(failure).trim(),
        ``,
        `This problem might be caused by ${context.siteName} changing its content/structure, in which case ${context.extensionName} needs to be updated accordingly. Otherwise, it's probably a bug in ${context.extensionName}.`,
        ``,
        `If you file a bug report, please include this message.`,
    ]);
}
exports.failureDescriber = failureDescriber;
